@@ -15,14 +15,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_ogg.c,v 1.177 2007/03/29 19:38:51 dgp85 Exp $
- *
  * demultiplexer for ogg streams
- *
  */
 /* 2003.02.09 (dilb) update of the handling for audio/video infos for strongarm cpus. */
 
@@ -236,13 +233,19 @@ static int64_t get_pts (demux_ogg_t *this, int stream_num , int64_t granulepos )
 static int read_ogg_packet (demux_ogg_t *this) {
   char *buffer;
   long bytes;
+  long total = 0;
   while (ogg_sync_pageout(&this->oy,&this->og)!=1) {
     buffer = ogg_sync_buffer(&this->oy, CHUNKSIZE);
     bytes  = this->input->read(this->input, buffer, CHUNKSIZE);
-    ogg_sync_wrote(&this->oy, bytes);
-    if (bytes < CHUNKSIZE/2) {
-      return 0;
+    if (bytes == 0) {
+      if (total == 0) {
+        lprintf("read_ogg_packet read nothing\n");
+        return 0;
+      }
+      break;
     }
+    ogg_sync_wrote(&this->oy, bytes);
+    total += bytes;
   }
   return 1;
 }
@@ -1218,16 +1221,17 @@ static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_pac
   case FLAC_BLOCKTYPE_STREAMINFO:
     _x_assert(header.length == FLAC_STREAMINFO_SIZE);
     _x_parse_flac_streaminfo_block(&op->packet[17], &streaminfo);
+
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, streaminfo.samplerate);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, streaminfo.channels);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, streaminfo.bits_per_sample);
+
     break;
   }
 
   this->si[stream_num]->buf_types = BUF_AUDIO_FLAC
     +this->num_audio_streams++;
 
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, streaminfo.samplerate);
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, streaminfo.channels);
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, streaminfo.bits_per_sample);
-
   this->si[stream_num]->factor = 90000;
 
   buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
@@ -1349,7 +1353,6 @@ static void send_header (demux_ogg_t *this) {
 
   while (!done) {
     if (!read_ogg_packet(this)) {
-      this->status = DEMUX_FINISHED;
       return;
     }
     /* now we've got at least one new page */
