@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine_decoder.c,v 1.48 2006/12/04 13:59:38 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * (ogg/)vorbis audio decoder plugin (libvorbis wrapper) for xine
  */
@@ -160,7 +158,7 @@ static void vorbis_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
       if( (res = vorbis_synthesis_headerin(&this->vi,&this->vc,&this->op)) < 0 ){ 
 	/* error case; not a vorbis header */
 	xine_log(this->stream->xine, XINE_LOG_MSG, "libvorbis: this bitstream does not contain vorbis audio data. Following first 64 bytes (return: %d).\n", res);
-	xine_hexdump(this->op.packet, this->op.bytes < 64 ? this->op.bytes : 64);
+	xine_hexdump((char *)this->op.packet, this->op.bytes < 64 ? this->op.bytes : 64);
 	return;
       }
 
@@ -221,7 +219,6 @@ static void vorbis_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
        */
 
       int i,j;
-      int clipflag=0;
       int bout=(samples<this->convsize?samples:this->convsize);
       audio_buffer_t *audio_buffer;
 
@@ -233,15 +230,13 @@ static void vorbis_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	ogg_int16_t *ptr=audio_buffer->mem+i;
 	float  *mono=pcm[i];
 	for(j=0;j<bout;j++){
-	  int val=mono[j]*32767.f;
+	  int val=(mono[j] + 1.0f) * 32768.f;
+          val -= 32768;
 	  /* might as well guard against clipping */
 	  if(val>32767){
 	    val=32767;
-	    clipflag=1;
-	  }
-	  if(val<-32768){
+	  } else if(val<-32768){
 	    val=-32768;
-	    clipflag=1;
 	  }
 	  *ptr=val;
 	  ptr+=this->vi.channels;
@@ -259,8 +254,9 @@ static void vorbis_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
       /* tell libvorbis how many samples we actually consumed */
       vorbis_synthesis_read(&this->vd,bout);
     }
-  } 
-  lprintf("output not open\n");
+  } else {
+    lprintf("output not open\n");
+  }
 }
 
 static void vorbis_dispose (audio_decoder_t *this_gen) {
